package clienteservicio;

import java.net.*;
import java.io.*;

import cargasintetica.Carga;
import cargasintetica.Cuentas_Calibracion;
import tiposcomunes.*;
import argparser.ArgParser;
import argparser.StringHolder;


public class Cliente {

	private ClienteGUI cg = null; 

	//Datos
	Integer nr_paquetes;
	FloatDistroData periodo = new FloatDistroData("Periodo (sg)");
	FloatDistroData tiempo_cpu = new FloatDistroData("Tiempo CPU (sg)");
	IntDistroData long_mensaje = new IntDistroData("Long mensaje (bytes)");
	DestData destino = new DestData("Destino");
	int id_calibracion = -1;

	Carga carga_sintetica = new Carga();
	
	LongComboData cuenta_calibracion = new LongComboData("Cuenta de calibracion", carga_sintetica);

	private static Cuentas_Calibracion cuentas_calibracion_conocidas = new Cuentas_Calibracion();

	String nombre = "";

	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		// Parseamos los argumentos con argparser
		StringHolder nr_paquetes = new StringHolder();
		StringHolder periodo = new StringHolder();
		StringHolder tiempo_cpu = new StringHolder();
		StringHolder long_mensaje = new StringHolder();
		StringHolder destino = new StringHolder();	
		StringHolder nombre = new StringHolder();
		StringHolder id_calibracion = new StringHolder();
		StringHolder cuentas_calibracion = new StringHolder();

		String str_ids_calibracion = cuentas_calibracion_conocidas.genera_str_ids_calibracion();
		ArgParser parser = new ArgParser("Cliente sintetico");

		parser.addOption("-nr_paquetes %s #nr_paquetes inyectados (0 ilimitado)", nr_paquetes);
		parser.addOption("-periodo %s #CTE|EXP|UNI|BIP,media,rango", periodo);
		parser.addOption("-tiempo_cpu %s #CTE|EXP|UNI|BIP,media,rango", tiempo_cpu);
		parser.addOption("-long_mensaje %s #CTE|EXP|UNI|BIP,bytes,rango", long_mensaje);
		parser.addOption("-destino %s #IPdest,Portdest", destino);
		parser.addOption("-nombre %s #Nombre del cliente", nombre);
		parser.addOption("-id_calibracion %s #" + str_ids_calibracion, id_calibracion);
		parser.addOption("-cuentas_calibracion %s #", cuentas_calibracion);
		parser.matchAllArgs(args);

		new Cliente(nr_paquetes, periodo, tiempo_cpu, long_mensaje, destino, nombre, id_calibracion, cuentas_calibracion);

	}




	public Cliente( StringHolder nr_paquetes_arg,
					StringHolder periodo_arg, StringHolder tiempo_cpu_arg, 
					StringHolder long_mensaje_arg, StringHolder destino_arg,
					StringHolder nombre_arg,
					StringHolder id_calibracion_arg,
					StringHolder cuentas_calibracion_arg)
	{
		cuenta_calibracion.load_hashmap(cuentas_calibracion_conocidas.exporta_hashmap());

		// Primero asignamos valores por defecto
		ValoresPorDefecto();

		if (nr_paquetes_arg.value != null)
		{
			nr_paquetes = Integer.valueOf(nr_paquetes_arg.value.trim()).intValue();
			
		}
		if (periodo_arg.value != null)
		{		
			periodo.parsea_cmdarg(periodo_arg.value);
		}

		if (tiempo_cpu_arg.value != null )
		{
			tiempo_cpu.parsea_cmdarg(tiempo_cpu_arg.value);
		}

		if (long_mensaje_arg.value != null)
		{

			long_mensaje.parsea_cmdarg(long_mensaje_arg.value);
			
		}

		if (destino_arg.value != null)
		{
			destino.parsea_cmdarg(destino_arg.value);
		}

		if (nombre_arg.value != null)
		{
			nombre = nombre_arg.value;
		}
		
		
		if (id_calibracion_arg.value != null)
		{
			cuenta_calibracion.parsea_cmdarg_ordinal(id_calibracion_arg.value);
		}
		
		if (cuentas_calibracion_arg.value != null)
		{
			cuenta_calibracion.parsea_cmdarg_value(cuentas_calibracion_arg.value);
		}

		if ( (id_calibracion_arg.value == null) && (cuentas_calibracion_arg.value == null) )
		{
			id_calibracion = -1;
			cuenta_calibracion.asigna_valor_por_id(id_calibracion);
		}

		// Creamos los threads y los arrancamos
		exec = new Executor();
		exec.start();


		cg = new ClienteGUI(this, nombre);
		cg.actualiza_datos_en_la_gui(nr_paquetes, periodo, tiempo_cpu, long_mensaje, destino);
		cg.arranca_gui();
		
	}
	
	
	
	private void ValoresPorDefecto()
	{
		nr_paquetes = 0;
		
		periodo.asigna_valores(TipoDistribucion.CONSTANTE, 1.0, 0.0);
		tiempo_cpu.asigna_valores(TipoDistribucion.CONSTANTE, 0.2, 0.0);
		long_mensaje.asigna_valores(TipoDistribucion.CONSTANTE, 256, 0);
		destino.asigna_valores("127.0.0.1", 12000);

		// Esto movido al constructor despues de parsear id_calibracion y cuenta_calibracion
		// para evitar una calibracion automatica
//		id_calibracion = -1;
//		cuenta_calibracion.asigna_valor_por_id(id_calibracion);
		
		
	}
	
	public void leeDatos() throws ConfiguracionNoValida
	{
		// Quick hack to pass integer by reference
		// cf http://stackoverflow.com/questions/3326112/java-best-way-to-pass-int-by-reference
		int nr_paquetes_ar[] = new int[1];
		cg.leedatos_de_la_gui(nr_paquetes_ar, periodo, tiempo_cpu, long_mensaje, destino);
		nr_paquetes = nr_paquetes_ar[0];
		chequeaDatos();
	}
	
	private void chequeaDatos() throws ConfiguracionNoValida
	{
		if (tiempo_cpu.nominal < 0)
		{
			throw new ConfiguracionNoValida();
		}
		// MORE TODO

	}
	
	

	private Executor exec = null;
	
	public void start_thread()
	{
		exec.reanuda();
	}
	
	public void stop_thread()
	{
		exec.pausa();
	}

	public void finaliza_thread()
	{
		exec.finaliza();
	}

	private class Executor extends ThreadControlado implements ActualizaCallback
	{
		DatagramSocket udpSocket=null;
		// Socket outputSocket; // Para TCP
		
		

		public Executor()
		{
			super.provee_actualiza_cb(this);

		}
		
		public void reanuda_callback()
		{
			System.out.println("CLIENTE " + nombre + ":  Reanuda la marcha");
			carga_sintetica.set_cuentas_calibracion(cuenta_calibracion.value);
		}

		public void antes_de_pausa_callback()
		{
			System.out.println("CLIENTE " + nombre + ":  Entro en pausa");
		}
		
		public void abort_callback()
		{
			
		}
		
		public void run()
		{
			
			boolean finalizado = false;
			
		
			System.out.println("CLIENTE " + nombre + ": inicializandose");
			// Si estuviesemos en TCP convendria abrir los sockets de salida
			try {
				udpSocket = new DatagramSocket();
			} catch (SocketException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			carga_sintetica.set_cuentas_calibracion(cuenta_calibracion.value);
			
			System.out.println("CLIENTE " + nombre + ": entra en bucle");

			int limite = 0;
			if (nr_paquetes > 0)
			{
				System.out.println("CLIENTE:  Voy a enviar " + nr_paquetes + " paquetes");
				limite = nr_paquetes;
			}
			else
			{
				limite = -1;
			}


			
			// Ciclo del Receptor
			while( ! finalizado)
			{
				// Esperamos a que nos segnalicen la salida (si no está bloqueado)
				boolean finalizo = this.chequea_estado_ejecucion();
				if (finalizo)
				{
					break;
				}

				long ts_inicio_ciclo_ns;
				long ts_despues_envio_ns;
				long ts_fin_ciclo_ns;
				long currentMesNumBytes;

				long tiempo_cpu_random_ns;
				long tiempo_periodo_random_ns;
				
				ts_inicio_ciclo_ns = System.nanoTime();

				//Evalua la carga de trabajo
				tiempo_cpu_random_ns = (long) tiempo_cpu.generaValorRandom();  // Devuelve ns (escala a 1E9)
				//System.out.println("tiempo random work : " + tiempo_cpu_random_ns);
				//Ejecuta la carga de trabajo
				ManejaTiempo.doWork(tiempo_cpu_random_ns, carga_sintetica);
				
				//Envia el mensaje
				currentMesNumBytes = long_mensaje.generaValorRandom();

				enviaMensaje(currentMesNumBytes, udpSocket);
				ts_despues_envio_ns = System.nanoTime();
				limite--;
				if (limite == 0)
				{
					System.out.println("CLIENTE " + nombre + ":  YA NO ENVIO MAS");
					break;
				}
				
				//Evalua el proximo inicio
				tiempo_periodo_random_ns = (long) periodo.generaValorRandom();  // Devuelve ns (escala a 1E9)
				//System.out.println("tiempo random periodo: " + tiempo_periodo_random_ns);
				
				long tiempo_sleep_milis = (long) ( (tiempo_periodo_random_ns - (ts_despues_envio_ns - ts_inicio_ciclo_ns))/1E6);
				//System.out.println("tiempo sleep milis: " + tiempo_sleep_milis);
				if (tiempo_sleep_milis > 0)
				{
					try {
						Thread.sleep(tiempo_sleep_milis);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				
				ts_fin_ciclo_ns = System.nanoTime();

				// Estadisticas
				long tiempo_work_ns = ts_despues_envio_ns - ts_inicio_ciclo_ns;
				long tiempo_ciclo_ns = ts_fin_ciclo_ns - ts_inicio_ciclo_ns;
				
				String tiempo_work_sg_y_milis;
				String tiempo_ciclo_sg_y_milis;
				tiempo_work_sg_y_milis =  ManejaTiempo.ns_to_sg_y_milis(tiempo_work_ns);
				tiempo_ciclo_sg_y_milis = ManejaTiempo.ns_to_sg_y_milis(tiempo_ciclo_ns);
				
				System.out.println("CLIENTE " + nombre + ":  CPU: " + tiempo_work_sg_y_milis + " sg   "
						           + "PERIODO: " + tiempo_ciclo_sg_y_milis + " sg    LONG MENSAJE: " + currentMesNumBytes + " bytes");
			}
		}
		
		
		private void enviaMensaje(long currentMesNumBytes, DatagramSocket udpSocket)
		{
			try{

				// Crea DatagramPacket
				ByteArrayOutputStream baos=new ByteArrayOutputStream();
				
				for (int i=0; i<currentMesNumBytes;i++) {
					baos.write((byte)(i %256));
				};
				DatagramPacket packet=new DatagramPacket(baos.toByteArray(),
						baos.toByteArray().length,
						InetAddress.getByName(destino.IP),
						destino.puerto);
				
				udpSocket.send(packet);

			} catch (IOException e) {e.printStackTrace();};
		}
		
	}


}
