package clienteservicio;

import java.util.EnumMap;
import java.util.EnumSet;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.events.ShellListener;


import tiposcomunes.ActionEnableDisableListener;
import tiposcomunes.IntDataGUI;
import tiposcomunes.LongComboData;
import tiposcomunes.LongComboDataGUI;
import tiposcomunes.TipoModo;
import tiposcomunes.DestData;
import tiposcomunes.DestDataGUI;
import tiposcomunes.FloatDistroData;
import tiposcomunes.FloatDistroDataGUI;
import tiposcomunes.IntDistroData;
import tiposcomunes.IntDistroDataGUI;
import tiposcomunes.EnumComboGUI;

//public class ClienteGUIData implements ClienteGUI
public class ClienteGUI
{
	private Cliente elCliente = null;
	
	private Shell shell = null;
	
	private IntDataGUI nr_paquetes_gui = null;
	
	private FloatDistroDataGUI periodo_gui = null;
	private FloatDistroDataGUI tiempo_cpu_gui = null;
	private IntDistroDataGUI long_mensaje_gui = null;
	private DestDataGUI destino_gui = null;
	private LongComboDataGUI cuentas_calibracion_gui = null;
	
	private Button boton_start;
	private Button boton_pausa;
	private enum Actions {START, PAUSA};
	private EnumSet<Actions> actions_enabled = EnumSet.noneOf(Actions.class);
	private EnumMap<Actions, Button> actions_buttons = new EnumMap<Actions, Button>(Actions.class);
	

	
	public ClienteGUI(Cliente elCliente, String app_name)
	{
		this.elCliente = elCliente;
		
		// Creamos la Shell con un GridLayout de 2 columnas
		this.shell = SWTUtil.getShell();
		shell.setText("Cliente " + app_name);
		shell.setLayout(new GridLayout(2, false) );
		

		// PRIMERA FILA:  Nr paquetes y botones
		nr_paquetes_gui = new IntDataGUI(shell, "Nr paquetes (0 ilimitado)");
		
		// Los botones los hacemos a pelo
		Group grupo_botones = new Group(shell, SWT.SHADOW_OUT);
		grupo_botones.setLayout(new GridLayout(2, false));

		boton_start = new Button(grupo_botones, SWT.TOGGLE);
		boton_start.setText("Start");
		boton_start.addSelectionListener(new StartButtonCallback() );
		actions_enabled.add(Actions.START);
		actions_buttons.put(Actions.START,  boton_start);
		
		boton_pausa = new Button(grupo_botones, SWT.TOGGLE);
		boton_pausa.setText("Pausa");
		// Incialmente el boton esta deshabilitado
		boton_pausa.setEnabled(false);
		boton_pausa.addSelectionListener(new PauseButtonCallback() );
		actions_buttons.put(Actions.PAUSA,  boton_pausa);
		
		
		// SEGUNDA FILA:  Periodo y longitud mensaje
		periodo_gui = new FloatDistroDataGUI(shell, elCliente.periodo);
		long_mensaje_gui = new IntDistroDataGUI(shell, elCliente.long_mensaje);

		// TERCERA FILA:  Tiempo CPU y destino
		tiempo_cpu_gui = new FloatDistroDataGUI(shell, elCliente.tiempo_cpu);
		destino_gui = new DestDataGUI(shell, elCliente.destino);
		
		// CUARTA FILA:  Calibracion
		cuentas_calibracion_gui = new LongComboDataGUI(shell, elCliente.cuenta_calibracion,
					   									new ActionEnableDisableHander() );
		
		shell.addShellListener(new EndGuiHandler());
		// Empacamos
		shell.pack();
		
	}
	
	// Alternativamente podemos heredar de SelectionAdapter
	private class StartButtonCallback implements SelectionListener
	{

		public void widgetSelected(SelectionEvent arg0)
		{
			try
			{
				elCliente.leeDatos();
				elCliente.start_thread();
				
				boton_start.setEnabled(false);
				boton_pausa.setEnabled(true);
				
				actions_enabled.clear();
				actions_enabled.add(Actions.PAUSA);
				
				cuentas_calibracion_gui.disable_actions();
				
			}
			catch (ConfiguracionNoValida e1)
			{
				e1.printStackTrace();
			}
			
		}

		// Typically called with double click
		public void widgetDefaultSelected(SelectionEvent arg0) 
		{
			// Do nothing (just to distinguish it)

			return;
			// widgetSelected(arg0)
		}
	}
	
	private class PauseButtonCallback implements SelectionListener
	{

		public void widgetSelected(SelectionEvent arg0)
		{
			elCliente.stop_thread();
				
			boton_start.setEnabled(true);
			boton_pausa.setEnabled(false);
			
			actions_enabled.clear();
			actions_enabled.add(Actions.START);
			
			cuentas_calibracion_gui.enable_actions();
		}

		// Typically called with double click
		public void widgetDefaultSelected(SelectionEvent arg0) 
		{
			// Do nothing (just to distinguish it)

			return;
			// widgetSelected(arg0)
		}
	}

	// ---------------------------------------------------------

	private class ActionEnableDisableHander implements ActionEnableDisableListener
	{

		public void disable_actions() 
		{
			Iterator<Actions> iterator =  actions_enabled.iterator();
			
			while(iterator.hasNext() )
			{
				Button boton = actions_buttons.get(iterator.next() );
				boton.setEnabled(false);
			}
			
		}

		public void enable_actions() 
		{
			Iterator<Actions> iterator =  actions_enabled.iterator();

			while(iterator.hasNext() )
			{
				Button boton = actions_buttons.get(iterator.next() );
				boton.setEnabled(true);
			}
			
		}
		
	}
	
	// ---------------------------------------------------------

	public void arranca_gui()
	{
		SWTUtil.openShell(shell);
	}

	// ---------------------------------------------------------

	public void leedatos_de_la_gui(int nr_paquetes_ar[], FloatDistroData periodo, 
			FloatDistroData tiempo_cpu, IntDistroData long_mensaje, DestData destino)
	
	{
		nr_paquetes_ar[0] =  nr_paquetes_gui.lee_valor();
		periodo_gui.lee_valor(periodo);
		tiempo_cpu_gui.lee_valor(tiempo_cpu);
		long_mensaje_gui.lee_valor(long_mensaje);
		destino_gui.lee_valor(destino);
		cuentas_calibracion_gui.lee_valor();
	}
	
	public void actualiza_datos_en_la_gui(int nr_paquetes, FloatDistroData periodo, 
			FloatDistroData tiempo_cpu, IntDistroData long_mensaje, DestData destino)
	{
		nr_paquetes_gui.actualiza_valor(nr_paquetes);
		periodo_gui.actualiza_valor(periodo);
		tiempo_cpu_gui.actualiza_valor(tiempo_cpu);
		long_mensaje_gui.actualiza_valor(long_mensaje);
		destino_gui.actualiza_valor(destino);
		cuentas_calibracion_gui.actualiza_valor();
	}

	// ---------------------------------------------------------

	private class EndGuiHandler implements ShellListener
	{

		@Override
		public void shellActivated(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		public void shellClosed(ShellEvent arg0) {

			elCliente.finaliza_thread();
			
		}

		@Override
		public void shellDeactivated(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shellDeiconified(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shellIconified(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}
		
	}
	
}
