package clienteservicio;



import java.io.*;
import java.net.*;
import java.util.*;

import cargasintetica.Carga;
import cargasintetica.Cuentas_Calibracion;


import tiposcomunes.DestPctData;
import tiposcomunes.EscuchaData;
import tiposcomunes.FloatDistroData;
import tiposcomunes.LongComboData;
import tiposcomunes.TipoDistribucion;
import tiposcomunes.TipoModo;

import argparser.ArgParser;
import argparser.StringHolder;

public class Servicio {

	private ServicioGUI sg = null;


	// Datos
	TipoModo modo = null;
	EscuchaData acceso = new EscuchaData("Recibiendo de:");
	DestPctData destino1 = new DestPctData("Destino 1");
	DestPctData destino2 = new DestPctData("Destino 2");
	FloatDistroData tiempo_cpu = new FloatDistroData("Tiempo CPU (sg)");
	int id_calibracion = -1;
	
	Carga carga_sintetica = new Carga();

	LongComboData cuenta_calibracion = new LongComboData("Cuenta de calibracion", carga_sintetica);

	private static Cuentas_Calibracion cuentas_calibracion_conocidas = new Cuentas_Calibracion();

	String nombre = "";
	
	
	private ReceptorMensajes receptor = null;
	private ProcesadorMensajes procesador = null;
	
	private ArrayList<MensajeLong> listaMensajes = new ArrayList<MensajeLong>();
	private Object listaMensajes_lock = new Object();
	

	
	public static void main(String[] args) {

		// Parseamos los argumentos con argparser
		StringHolder modo = new StringHolder();
		StringHolder acceso = new StringHolder();
		StringHolder tiempo_cpu = new StringHolder();
		StringHolder destino1 = new StringHolder();
		StringHolder destino2 = new StringHolder();
		StringHolder nombre = new StringHolder();
		StringHolder id_calibracion = new StringHolder();
		StringHolder cuentas_calibracion = new StringHolder();
		
		String str_ids_calibracion = cuentas_calibracion_conocidas.genera_str_ids_calibracion();
		ArgParser parser = new ArgParser("Servicio sintetico");
		
		parser.addOption("-modo %s #EXCL|NOEXCL", modo);
		parser.addOption("-acceso %s #puerto", acceso);
		parser.addOption("-tiempo_cpu %s #CTE|EXP|UNI|BIP,media,rango", tiempo_cpu);
		parser.addOption("-destino1 %s #Porcentage,IPdest,Portdest", destino1);
		parser.addOption("-destino2 %s #Porcentage,IPdest,Portdest", destino2);
		parser.addOption("-nombre %s #Nombre del servicio", nombre);
		parser.addOption("-id_calibracion %s #" + str_ids_calibracion, id_calibracion);
		parser.addOption("-cuentas_calibracion %s #", cuentas_calibracion);
		
		parser.matchAllArgs(args);

		new Servicio(modo, acceso, tiempo_cpu, destino1, destino2, nombre, 
					id_calibracion, cuentas_calibracion);
		System.exit(0);
	}

	
	
	public Servicio(StringHolder modo_arg,
					StringHolder acceso_arg, StringHolder tiempo_cpu_arg, 
			        StringHolder destino1_arg, StringHolder destino2_arg,
			        StringHolder nombre_arg, StringHolder id_calibracion_arg,
			        StringHolder cuentas_calibracion_arg)
	{
		cuenta_calibracion.load_hashmap(cuentas_calibracion_conocidas.exporta_hashmap());
		
		// Primero asignamos valores por defecto
		valores_por_defecto();
		
		if (modo_arg.value != null)
		{
			if ( modo_arg.value.equals("EXCL") )
			{
				modo = TipoModo.EXCLUYENTE;
			}
			else if ( modo_arg.value.equals("NOEXCL") )
			{
				modo = TipoModo.NO_EXCLUYENTE;
			}
			
		}
		
		if (acceso_arg.value != null)
		{
			acceso.parsea_cmdarg(acceso_arg.value);
		}
		
		if (tiempo_cpu_arg.value != null)
		{
			tiempo_cpu.parsea_cmdarg(tiempo_cpu_arg.value);
		}
		
		if (destino1_arg.value != null)
		{
			destino1.parsea_cmdarg(destino1_arg.value);
		}
		
		if (destino2_arg.value != null)
		{
			destino2.parsea_cmdarg(destino2_arg.value);
		}
		
		if (nombre_arg.value != null)
		{
			nombre = nombre_arg.value;
		}

		if (id_calibracion_arg.value != null)
		{
			cuenta_calibracion.parsea_cmdarg_ordinal(id_calibracion_arg.value);
		}
		
		if (cuentas_calibracion_arg.value != null)
		{
			cuenta_calibracion.parsea_cmdarg_value(cuentas_calibracion_arg.value);
		}

		if ( (id_calibracion_arg.value == null) && (cuentas_calibracion_arg.value == null) )
		{
			id_calibracion = -1;
			cuenta_calibracion.asigna_valor_por_id(id_calibracion);
		}
		
		
		
		// Creamos los threads y los arrancamos
		receptor = new ReceptorMensajes();
		procesador = new ProcesadorMensajes();
		
		receptor.start();
		procesador.start();
		
		sg = new ServicioGUI(this, nombre);
		sg.actualiza_datos_en_la_gui(modo, acceso, tiempo_cpu, destino1, destino2);
		sg.arranca_gui();
	}
	
	
	private void valores_por_defecto()
	{
		modo = TipoModo.EXCLUYENTE;

		acceso.asigna_valores(12000);
		tiempo_cpu.asigna_valores(TipoDistribucion.CONSTANTE, 0.2, 0.0);
		destino1.asigna_valores("127.0.0.1", 11000, 0);
		destino2.asigna_valores("127.0.0.1", 13000, 0);

		// Esto movido al constructor despues de parsear id_calibracion y cuenta_calibracion
		// para evitar una calibracion automatica
//		id_calibracion = -1;
//		cuenta_calibracion.asigna_valor_por_id(id_calibracion);

	}


	// Esta funcion es llamada por la GUI cuando se da a start
	public void leeDatos() throws ConfiguracionNoValida
	{
		sg.leedatos_de_la_gui(modo, acceso, tiempo_cpu, destino1, destino2);
		chequeaDatos();
	}

	private void chequeaDatos() throws ConfiguracionNoValida
	{
		if (modo == TipoModo.EXCLUYENTE)
			{
				if (destino1.porcentage + destino2.porcentage > 100)
				{
					throw new ConfiguracionNoValida();
				}
			}
		// TODO more checks
	}

	public void start_thread()
	{
		procesador.reanuda();
		receptor.reanuda();
	}

	public void stop_thread()
	{
		receptor.pausa();
		procesador.pausa();
	}

	public void finaliza_thread()
	{
		receptor.finaliza();
		procesador.finaliza();
	}
	
	
	private class MensajeLong
	{
		public int numMensaje;
		public int lenMensaje;
		
		public MensajeLong(int numMensaje, int lenMensaje)
		{
			this.numMensaje = numMensaje;
			this.lenMensaje = lenMensaje;
			// Se puede poner tambien el origen
			// TODO:  Poner el timestamp
		}
	}
	
	// Thread que se dedica a leer los mensajes y producir entradas en listaMensajes
	private class ReceptorMensajes extends ThreadControlado implements ActualizaCallback
	{
		private DatagramSocket socket_entrada = null;

		public ReceptorMensajes()
		{
			super.provee_actualiza_cb(this);

		}
		public void reanuda_callback()
		{
			System.out.println("RECEPTOR " + nombre +": Reanuda la marcha");
			
			// Cerramos el socket_entrada y lo reabrimos con los nuevos datos
			if ( (socket_entrada != null) && (! socket_entrada.isClosed()) )
			{
				socket_entrada.close();
			}
			try {
				socket_entrada = new DatagramSocket(acceso.puerto);
			} catch (SocketException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		
		public void antes_de_pausa_callback()
		{
			System.out.println("RECEPTOR " + nombre + ": Entro en pausa");
		}
		
		public void abort_callback()
		{
			// Cerramos el socket para quitar el blockeo en el receive
			// NOTA:  Saltara una excepcion
			if (socket_entrada != null)
			{
				socket_entrada.close();
			}
			
		}
		public void run()
		{
			// Abrimos el socket de entrada
			System.out.println("RECEPTOR " + nombre + ": inicializandose");
			
			byte[] udpInputBuffer = new byte[65536];
			int numMensaje = 0;

			System.out.println("RECEPTOR " + nombre + ": entra en bucle");
			// Ciclo del Receptor
			while(true)
			{
				// Esperamos a que nos segnalicen la salida (si no está bloqueado)
				boolean finalizo = this.chequea_estado_ejecucion();
				if (finalizo)
				{
					// Terminamos pero mandamos un thread notify al procesador para que
					// el tambien finalice
					synchronized(listaMensajes_lock)
					{
						// NOTIFICAMOS AL PROCESADOR
						// He de meter un mensaje dummy para que el procesador salga
						// del while
						MensajeLong mensalong_dummy = new MensajeLong(numMensaje, 42);
						listaMensajes.add(mensalong_dummy);
						listaMensajes_lock.notify();
					}
					break;
				}
				// Leemos 1 datagrama
				DatagramPacket recPacket = new DatagramPacket(udpInputBuffer, 65536);
				try {
					
					socket_entrada.receive(recPacket);
				} catch (SocketException e)
				{
					// Detectamos si nos han cerrado el socket en el callback abort
					if (socket_entrada.isClosed())
					{
						// Ok nos han abortado, pero NO despertamos al thread procesador
						continue;
					}
				}
				catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				int longMensaje = recPacket.getLength();
//				System.out.println("RECEPTOR " + nombre
//									+ ": encolando mensaje No " + numMensaje 
//									+ " de " + longMensaje + " bytes");
				MensajeLong mensalong = new MensajeLong(numMensaje, longMensaje);
				
				synchronized(listaMensajes_lock)
				{
					listaMensajes.add(mensalong);
					listaMensajes_lock.notify();
				}

				// Aumentamos el contador de mensajes
				numMensaje++;
			}
			
		}
		
	}
	
	private class ProcesadorMensajes extends ThreadControlado implements ActualizaCallback
	{
		DatagramSocket udpSocket1 = null;
		DatagramSocket udpSocket2 = null;
		
		public ProcesadorMensajes()
		{
			super.provee_actualiza_cb(this);

		}
		
		public void reanuda_callback()
		{
			carga_sintetica.set_cuentas_calibracion(cuenta_calibracion.value);
			System.out.println("PROCESADOR " + nombre + ":  Reanuda la marcha en modo " + modo.toString());
		}
		
		public void abort_callback()
		{
			
		}

		public void antes_de_pausa_callback()
		{
			System.out.println("PROCESADOR " + nombre + ":  Entro en pausa");
		}
		public void run()
		{
			System.out.println("PROESADOR " + nombre + ": inicializandose");
			// Si estuviesemos en TCP convendria abrir los sockets de salida pero habria
			// que hacerlo en los reanuda y pausa ya que los destinos pueden haber cambiado
			try {
				udpSocket1 = new DatagramSocket();
				udpSocket2 = new DatagramSocket();
			} catch (SocketException e1) {
				// Al no pedir un puerto en concreto en principio no deberia de dar
				// problemas.
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			MensajeLong el_mensaje = null;
			System.out.println("PROCESADOR " + nombre + ": entra en bucle");

			// Ciclo del Receptor
			while(true)
			{
				// Esperamos a que nos segnalicen la salida (si no está bloqueado)
				boolean finalizo = this.chequea_estado_ejecucion();
				if (finalizo)
				{
					break;
				}

				// Miramos la cola de mensajes
				synchronized (listaMensajes_lock)
				{
					// Mientras no haya mensajes esperamos
					while(listaMensajes.size() == 0)
					{
						try {
							listaMensajes_lock.wait();
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
					
					// Volvemos a mirar si nos han finalizado durante el wait
					finalizo = this.chequea_estado_ejecucion();
					if (finalizo)
					{
						break;
					}

					
					// Extraemos el primer mensaje
					el_mensaje = listaMensajes.remove(0);
				}

				long tiempo_cpu_random_ns;
				long ts_inicio_ciclo_ns;
				long ts_fin_ciclo_ns;

				ts_inicio_ciclo_ns = System.nanoTime();
				
//				System.out.println("PROCESADOR " + nombre + 
//							":  Desencolando mensaje No " + el_mensaje.numMensaje 
//							+ " de " + el_mensaje.lenMensaje + " bytes");

				//Evalua la carga de trabajo
				tiempo_cpu_random_ns = (long) tiempo_cpu.generaValorRandom();  // Devuelve ns (escala a 1E9)
				//System.out.println("tiempo random work : " + tiempo_cpu_random_ns);
				//Ejecuta la carga de trabajo
				ManejaTiempo.doWork(tiempo_cpu_random_ns, carga_sintetica);


				
				ResultadoSorteo res = null;
				switch(modo)
				{
				case EXCLUYENTE:
					res = sorteo_excluyente();
					break;
					
				case NO_EXCLUYENTE:
					res = sorteo_no_excluyente();
					break;
				
				}
				
				if (res == ResultadoSorteo.DESTINO1 || res == ResultadoSorteo.AMBOS)
				{
					envia_mensaje(el_mensaje, destino1, udpSocket1);
				}
				
				if (res == ResultadoSorteo.DESTINO2 || res == ResultadoSorteo.AMBOS)
				{
					envia_mensaje(el_mensaje, destino2, udpSocket2);
				}
				
				ts_fin_ciclo_ns = System.nanoTime();
				
				// Estadisticas
				long tiempo_ciclo_ns = ts_fin_ciclo_ns - ts_inicio_ciclo_ns;
				String tiempo_ciclo_sg_y_milis;
				tiempo_ciclo_sg_y_milis = ManejaTiempo.ns_to_sg_y_milis(tiempo_ciclo_ns);
				
				System.out.println("SERVIDOR " + nombre + ":  WORK: " + tiempo_ciclo_sg_y_milis + " sg" 
						+ " enviado a " + res.toString());
				
			}
			
			
		}

		private void envia_mensaje(MensajeLong elMensaje, DestPctData destino, DatagramSocket udpSocket)
		{
			// Crea DatagramPacket
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			int len_mensaje = elMensaje.lenMensaje;
			
						
			for (int i = 0  ; i < len_mensaje ; i++)
			{
				baos.write((byte)(i %256));
			}
			DatagramPacket packet;
			try {
				packet = new DatagramPacket(baos.toByteArray(),
											baos.toByteArray().length,
											InetAddress.getByName(destino.IP),
											destino.puerto);
				udpSocket.send(packet);
			} 
			catch (UnknownHostException e1) 
			{
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}
			
			
	}
	
	// Thread que se dedica a leer los mensajes y producir entradas en listaMensajes
	enum ResultadoSorteo {DESTINO1, DESTINO2, PERDIDO, AMBOS};
	
	private ResultadoSorteo sorteo_excluyente()
	{
		ResultadoSorteo res;
		
		double sorteo_normalizado;
		double sorteo_pct;
		
		// Se sortea solo 1 vez
		sorteo_normalizado = RandomGenerator.normalizedUniformValue();
		sorteo_pct = sorteo_normalizado * 100;
		
		if (sorteo_pct < destino1.porcentage)
		{
			res = ResultadoSorteo.DESTINO1;
		}
		else if (sorteo_pct < destino1.porcentage + destino2.porcentage)
		{
			res = ResultadoSorteo.DESTINO2;
		}
		else
		{
			res = ResultadoSorteo.PERDIDO;
		}
		
		return res;
	}
	
	private ResultadoSorteo sorteo_no_excluyente()
	{
		ResultadoSorteo res;
		
		double sorteo_normalizado1, sorteo_normalizado2;
		double sorteo_pct1, sorteo_pct2;
		
		sorteo_normalizado1 = RandomGenerator.normalizedUniformValue();
		sorteo_pct1 = sorteo_normalizado1 * 100;
		
		sorteo_normalizado2 = RandomGenerator.normalizedUniformValue();
		sorteo_pct2 = sorteo_normalizado2 * 100;
		
		
		if (sorteo_pct1 < destino1.porcentage)
		{
			if (sorteo_pct2 < destino2.porcentage)
			{
				res = ResultadoSorteo.AMBOS;
			}
			else
			{
				res = ResultadoSorteo.DESTINO1;
			}
		}
		else
		{
			if (sorteo_pct2 < destino2.porcentage)
			{
				res = ResultadoSorteo.DESTINO2;
			}
			else
			{
				res = ResultadoSorteo.PERDIDO;
			}
		}
		
		return res;
	}
	
	



}
