package clienteservicio;

import java.util.EnumMap;
import java.util.EnumSet;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.events.ShellListener;


import tiposcomunes.ActionEnableDisableListener;
import tiposcomunes.DestPctData;
import tiposcomunes.DestPctDataGUI;
import tiposcomunes.EnumComboGUI;
import tiposcomunes.EscuchaData;
import tiposcomunes.EscuchaDataGUI;
import tiposcomunes.FloatDistroData;
import tiposcomunes.FloatDistroDataGUI;
import tiposcomunes.LongComboDataGUI;
import tiposcomunes.TipoModo;



public class ServicioGUI 
{
	private Servicio elServicio;
	
	private Shell shell = null;
	
	private EnumComboGUI combo_gui;
	private EscuchaDataGUI acceso_gui;
	private FloatDistroDataGUI tiempo_cpu_gui;
	private DestPctDataGUI destino1_gui;
	private DestPctDataGUI destino2_gui;
	private LongComboDataGUI cuentas_calibracion_gui = null;
	
	
	private Button boton_start;
	private Button boton_pausa;
	private enum Actions {START, PAUSA};
	private EnumSet<Actions> actions_enabled = EnumSet.noneOf(Actions.class);
	private EnumMap<Actions, Button> actions_buttons = new EnumMap<Actions, Button>(Actions.class);

	
	public ServicioGUI(Servicio elServicio, String app_name)
	{
		this.elServicio = elServicio;
		
		// Creamos la Shell con un GridLayout de 2 columnas
		this.shell = SWTUtil.getShell();
		shell.setText("Servicio " + app_name);
		shell.setLayout(new GridLayout(2, false) );
		
		
		// Primera fila:  Grupo MODO y botones
		combo_gui = new EnumComboGUI(shell, elServicio.modo, "Modo");
		
		// Los botones los hacemos a pelo en un grupo
		Group grupo_botones = new Group(shell, SWT.SHADOW_OUT);
		grupo_botones.setLayout(new GridLayout(2, false) );
		
		boton_start = new Button(grupo_botones, SWT.TOGGLE);
		boton_start.setText("Start");
		boton_start.addSelectionListener(new StartButtonCallback());
		actions_enabled.add(Actions.START);
		actions_buttons.put(Actions.START,  boton_start);

		
		boton_pausa = new Button(grupo_botones, SWT.TOGGLE);
		boton_pausa.setText("Pausa");
		boton_pausa.setEnabled(false);
		boton_pausa.addSelectionListener(new PausaButtonCallback() );
		actions_buttons.put(Actions.PAUSA,  boton_pausa);

		
		// Segunda fila:  Entrada y Destino 1
		acceso_gui = new EscuchaDataGUI(shell, elServicio.acceso);
		destino1_gui = new DestPctDataGUI(shell, elServicio.destino1);
		
		// Tercera fila:  tiempo_cpu y Destino 2
		tiempo_cpu_gui = new FloatDistroDataGUI(shell, elServicio.tiempo_cpu);
		destino2_gui = new DestPctDataGUI(shell, elServicio.destino2);
		
		// CUARTA FILA:  Calibracion
		cuentas_calibracion_gui = new LongComboDataGUI(shell, elServicio.cuenta_calibracion,
					new ActionEnableDisableHander() );
				
		
		// End of GUI handler
		shell.addShellListener(new EndGuiHandler());
		
		// Empacamos
		shell.pack();
	}
	
	private class StartButtonCallback extends SelectionAdapter
	{
		public void widgetSelected(SelectionEvent e)
		{
			try
			{
				elServicio.leeDatos();
				elServicio.start_thread();
				
				boton_start.setEnabled(false);
				boton_pausa.setEnabled(true);
				
				actions_enabled.clear();
				actions_enabled.add(Actions.PAUSA);
				
				cuentas_calibracion_gui.disable_actions();
				
			}
			catch( ConfiguracionNoValida e1)
			{
				e1.printStackTrace();
			}
		}
	}

	private class PausaButtonCallback extends SelectionAdapter
	{
		public void widgetSelected(SelectionEvent e)
		{
			//System.out.println("me pausan");
			elServicio.stop_thread();

			boton_start.setEnabled(true);
			boton_pausa.setEnabled(false);
			
			actions_enabled.clear();
			actions_enabled.add(Actions.START);
			
			cuentas_calibracion_gui.enable_actions();
			
		}
	}
	
	public void arranca_gui()
	{
		SWTUtil.openShell(shell);
	}

	public void leedatos_de_la_gui(TipoModo modo, EscuchaData acceso, 
									FloatDistroData tiempo_cpu,
									DestPctData destino1, DestPctData destino2)
	{
		combo_gui.lee_valor(modo);
		acceso_gui.lee_valor(acceso);
		tiempo_cpu_gui.lee_valor(tiempo_cpu);
		destino1_gui.lee_valor(destino1);
		destino2_gui.lee_valor(destino2);
		cuentas_calibracion_gui.lee_valor();

	}
	
	public void actualiza_datos_en_la_gui(TipoModo modo, EscuchaData acceso,
										FloatDistroData tiempo_cpu,
										DestPctData destino1, DestPctData destino2)
	{
		combo_gui.actualiza_valor(modo);
		acceso_gui.actualiza_valor(acceso);
		tiempo_cpu_gui.actualiza_valor(tiempo_cpu);
		destino1_gui.actualiza_valor(destino1);
		destino2_gui.actualiza_valor(destino2);
		cuentas_calibracion_gui.actualiza_valor();

	}
	
	private class ActionEnableDisableHander implements ActionEnableDisableListener
	{

		public void disable_actions() 
		{
			Iterator<Actions> iterator =  actions_enabled.iterator();
			
			while(iterator.hasNext() )
			{
				Button boton = actions_buttons.get(iterator.next() );
				boton.setEnabled(false);
				
				
			}
			
		}

		public void enable_actions() 
		{
			Iterator<Actions> iterator =  actions_enabled.iterator();

			while(iterator.hasNext() )
			{
				Button boton = actions_buttons.get(iterator.next() );
				boton.setEnabled(true);
			}
			
		}
		
	}
	
	// -----------------------------------------------------------------
	
	private class EndGuiHandler implements ShellListener
	{

		@Override
		public void shellActivated(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		public void shellClosed(ShellEvent arg0) {

			elServicio.finaliza_thread();
			
		}

		@Override
		public void shellDeactivated(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shellDeiconified(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void shellIconified(ShellEvent arg0) {
			// TODO Auto-generated method stub
			
		}
	}


}