package clienteservicio;



public class ThreadControlado extends Thread
{
	enum EstadoEjecucion {PAUSADO, CORRIENDO, FINALIZADO};

	private EstadoEjecucion estado_ejecucion = EstadoEjecucion.PAUSADO;
	private Object estado_ejecucion_lock = new Object();
	private ActualizaCallback actualiza_cb;
			
	protected void provee_actualiza_cb(ActualizaCallback actualiza_cb)
	{
		this.actualiza_cb = actualiza_cb;
	}
	
	protected boolean chequea_estado_ejecucion()
	{
		
		synchronized (estado_ejecucion_lock)
		{
			while (estado_ejecucion != EstadoEjecucion.CORRIENDO)
			{
				if (estado_ejecucion == EstadoEjecucion.FINALIZADO)
				{
					// Hemos finalizado
					return true;
				}
				
				assert(estado_ejecucion == EstadoEjecucion.PAUSADO);
				
				actualiza_cb.antes_de_pausa_callback();
				try {
					estado_ejecucion_lock.wait();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				// El finalizado podria haber llegado durante el wait
				if (estado_ejecucion == EstadoEjecucion.FINALIZADO)
				{
					// Hemos finalizado
					return true;
				}
				
				// Llamamos al callback de actualizacion
				actualiza_cb.reanuda_callback();
			}
			// No hemos finalizado
			return false;
		}
	}
	
	public void pausa()
	{
		synchronized(estado_ejecucion_lock)
		{
			estado_ejecucion = EstadoEjecucion.PAUSADO;
		}
		actualiza_cb.abort_callback();
	}
	
	public void reanuda()
	{
		synchronized(estado_ejecucion_lock)
		{
			estado_ejecucion = EstadoEjecucion.CORRIENDO;
			estado_ejecucion_lock.notify();
		}
	}
	
	public void finaliza()
	{
		synchronized(estado_ejecucion_lock)
		{
			if (estado_ejecucion == EstadoEjecucion.PAUSADO)
			{
				estado_ejecucion_lock.notify();
			}
			else if (estado_ejecucion == EstadoEjecucion.CORRIENDO)
			{
				actualiza_cb.abort_callback();
			}
			estado_ejecucion = EstadoEjecucion.FINALIZADO;
		}

	}
}