// Feature-test-macro  (see man 7 feature_test_macros   Required for syscall(gettid)
#define _GNU_SOURCE

#include <jni.h>
#include "cpuclock_CpuClock.h"

#include <stdio.h>
#include <errno.h>
#include <pthread.h>
#include <time.h>


#define DEBUGUEANDO 1


/*
 * Class:     cpuclock_CpuClock
 * Method:    get_thread_clock_id
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_cpuclock_CpuClock_get_1thread_1clock_1id(JNIEnv *env, jobject obj)
{
    clockid_t thread_clock_id;
    int res;

    res = pthread_getcpuclockid( pthread_self(), &thread_clock_id );
    if (res != 0)
    {
#ifdef DEBUGUEANDO
        perror("pthread_getcpuclockid");
#endif
        return 0;  // An error has occured
    }

    return (jlong) thread_clock_id;
}

/*
 * Class:     cpuclock_CpuClock
 * Method:    get_process_clock_id
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_cpuclock_CpuClock_get_1process_1clock_1id(JNIEnv *env, jobject obj)
{
    clockid_t process_clock_id;
    int res;

    res = clock_getcpuclockid(getpid(), &process_clock_id);
    if (res != 0)
    {
#ifdef DEBUGUEANDO
        perror("clock_getcpuclockid");
#endif
        return 0; // An error has occured
    }

    return (jlong) process_clock_id;
}

/*
 * Class:     cpuclock_CpuClock
 * Method:    clock_get_time_tspec
 * Signature: (J[J)V
 */
JNIEXPORT void JNICALL Java_cpuclock_CpuClock_clock_1get_1time_1tspec(JNIEnv *env, jobject obj, jlong clock, 
                                                                      jlongArray time_tspec)
{
    clockid_t clock_id;
    struct timespec ts;

    jclass cpu_clock_class;
    jfieldID clock_id_field;
    jlong *times_array;
    jsize array_size;

    int res;


    // Obtenemos una referencia al array
    times_array = (*env)->GetLongArrayElements(env, time_tspec, NULL);

    // Verificamos que tiene tamagno 2
    array_size = (*env)->GetArrayLength(env, time_tspec);
    if (array_size != 2)
    {
#ifdef DEBUGUEANDO
        printf("Tamagno del array distinto del previsto\n");
#endif
        // Liberamos el array descartando cambios
        (*env)->ReleaseLongArrayElements(env, time_tspec, times_array, JNI_ABORT);
        return;
    }

    clock_id = (clockid_t) clock;
    res = clock_gettime(clock_id, &ts);
    if (res != 0)
    {
        // Ha habido un error, ponemos 0 en el times array
        perror("clock_gettime");

        times_array[0] = (jlong) 0;
        times_array[1] = (jlong) 0;
    }
    else
    {
        // Success
        times_array[0] = (jlong) ts.tv_nsec;
        times_array[1] = (jlong) ts.tv_sec;
    }

    // Liberamos el array actualizando los cambios (ultimo argumento 0)
    (*env)->ReleaseLongArrayElements(env, time_tspec, times_array, 0);

}    

