// Feature-test-macro  (see man 7 feature_test_macros   Required for syscall(gettid)
#define _GNU_SOURCE

#include <jni.h>
#include "cpuclock_CpuClock.h"


#include <sys/types.h>
#include <linux/unistd.h>
#include <pthread.h>
#include <time.h>

#include <sys/syscall.h> // For SYS_gettid

#define DEBUGUEANDO 1



/*
 * Class:     cpuclock_CpuClock
 * Method:    get_clock_id
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_cpuclock_CpuClock_get_1clock_1id(JNIEnv *env, jobject obj)
{
    pid_t tid;
    clockid_t pthread_clock_id;
    clockid_t tid_clock_id = 0;

    jclass cpu_clock_class;
    jfieldID tid_field;

    int res;

    cpu_clock_class = (*env)->GetObjectClass(env, obj);

    tid_field = (*env)->GetFieldID(env, cpu_clock_class, "tid", "J");
    if (tid_field == NULL)
    {
        return (jlong) 0L; // Excepcion lanzada
    }

    tid = (pid_t) (*env)->GetLongField(env, obj, tid_field);
    
    res = pthread_getcpuclockid(pthread_self(), &pthread_clock_id);
    if (res != 0)
    {
#ifdef DEBUGUEANDO
        printf("pthread_getcpuclockid falla\n");
#endif
        return (jlong) 0L;
    }

/*
    res = clock_getcpuclockid(tid, &tid_clock_id);
    if (res != 0)
    {
#ifdef DEBUGUEANDO
        printf("clock_getcpuclockid falla\n");
#endif
        return (jlong) 0L;
    }

*/
    
#ifdef DEBUGUEANDO
    printf("TID cpuclock_id: %lu     PTHREAD cpuclock_id:  %lu\n",  tid_clock_id,  pthread_clock_id);
#endif
    
    return (jlong) pthread_clock_id;
}

/*
 * Class:     cpuclock_CpuClock
 * Method:    clock_cpu_get_time
 * Signature: ([J)V
 */
JNIEXPORT void JNICALL Java_cpuclock_CpuClock_clock_1cpu_1get_1time(JNIEnv *env, jobject obj, jlongArray times)
{
    clockid_t clock_id;
    struct timespec ts;

    jclass cpu_clock_class;
    jfieldID clock_id_field;
    jlong *times_array;
    jsize array_size;

    int res;

    
    cpu_clock_class = (*env)->GetObjectClass(env, obj);

    clock_id_field = (*env)->GetFieldID(env, cpu_clock_class, "clock_id", "J");
    if (clock_id_field == NULL)
    {
        return; // Excepcion lanzada
    }

    clock_id = (clockid_t) (*env)->GetLongField(env, obj, clock_id_field);

    res = clock_gettime(clock_id, &ts);

    // Obtenemos una referencia al array
    times_array = (*env)->GetLongArrayElements(env, times, NULL);

    // Verificamos que tiene tamagno 2
    array_size = (*env)->GetArrayLength(env, times);
    if (array_size != 2)
    {
        printf("Tamagno del array distinto del previsto\n");

        // Liberamos el array descartando cambios
        (*env)->ReleaseLongArrayElements(env, times, times_array, JNI_ABORT);
    }

    times_array[0] = (jlong) ts.tv_nsec;
    times_array[1] = (jlong) ts.tv_sec;

    // Liberamos el array actualizando los cambios (ultimo argumento 0)
    (*env)->ReleaseLongArrayElements(env, times, times_array, 0);
    
    return;
}

