#!/usr/bin/perl -w
use strict;

use XML::DOM::XPath;
use Proc::Background;


# Configuracion de la instalacion de Java
# ---------------------------------------
#my $main_classdir = "/home/miguel/workspace_dec/LoadProyect/bin";
#my $main_classdir = "/home/miguel/temp/egit/LoadProject/LoadProyect/bin";
my $main_classdir = "/home/alumnos/workspace/LoadProyect/bin";
#my $main_classdir = "/home/miguel/workspace_dec/LoadProyect/bin";
my $swt_jar = "/usr/share/java/swt.jar";
my $java_defines = "-Dfile.encoding=UTF-8";




# Function prototypes
# -------------------
sub construye_cliente_cmdline;
sub launch_cmdline;


# -----------------
# CMDLINE ARGUMENTS
# -----------------
my $filename;
my $cmd_nodo;
my $cmd_nombre;



if (scalar(@ARGV) == 2)
{
    $filename = $ARGV[0];
    $cmd_nodo = $ARGV[1];
    $cmd_nombre = "";
}
elsif (scalar(@ARGV) == 3)
{
    $filename = $ARGV[0];
    $cmd_nodo = $ARGV[1];
    $cmd_nombre = $ARGV[2];
}
else
{
    die("Usage: $0 <filename> <conf_nbr> [cmd_nombre]\n");
}


# Parseamos el fichero con XML::DOM
# ---------------------------------
my $parser = new XML::DOM::Parser;
my $tree = $parser->parsefile($filename);


# Sacamos los nodos y los listeners
# ---------------------------------

my %nodos_ip;
my %listener_port;



my @nodos_top = $tree->findnodes('/sistema/nodos');
if ( scalar(@nodos_top) != 1)
{
    die("Solo puede haber un único elemento /sistema/nodos");
}

my @lista_nodos = $nodos_top[0]->getChildNodes();

for my $nodo (@lista_nodos)
{
    if ($nodo->getNodeType() != XML::DOM::ELEMENT_NODE)
    {
        # Metadata de DOM
        next;
    }

    my $id = $nodo->getAttributeNode("id")->getValue();
    my $ip = $nodo->getAttributeNode("ip")->getValue();

    $nodos_ip{$id} = $ip;
}


my @listeners_top = $tree->findnodes('/sistema/listeners');
if ( scalar(@listeners_top) != 1)
{
    die("Solo puede haber un único elemento /sistema/listeners");
}

my @lista_listeners = $listeners_top[0]->getChildNodes();

for my $listener (@lista_listeners)
{
    if ($listener->getNodeType() != XML::DOM::ELEMENT_NODE)
    {
        # Metadata de DOM
        next;
    }

    my $id = $listener->getAttributeNode("id")->getValue();
    my $puerto = $listener->getAttributeNode("puerto")->getValue();

    $listener_port{$id} = $puerto;
}


# Leemos la config que nos toca
# -----------------------------
my @configs_top = $tree->findnodes('/sistema/configs');
if ( scalar(@listeners_top) != 1)
{
    die("Solo puede haber un único elemento /sistema/configs");
}

my @lista_configs = $configs_top[0]->getChildNodes();

foreach my $config (@lista_configs)
{
    if ($config->getNodeType() != XML::DOM::ELEMENT_NODE)
    {
        # Metadata de DOM
        next;
    }



    my $nodo = $config->getAttributeNode("nodo")->getValue();

    if ($nodo ne $cmd_nodo)
    {
        next; # No es el nodo especificado en el cmdline
    }

    my @lista_apps = $config->getChildNodes();

    foreach my $app (@lista_apps)
    {

        if ($app->getNodeType() != XML::DOM::ELEMENT_NODE)
        {
            # Metadata de DOM
            next;
        }


        my $nombre = $nodo . "-" . $app->getAttributeNode("id")->getValue();

        if  ( ($cmd_nombre ne "") && ($nombre ne $cmd_nombre) )
        {
            next;  # No es la app especificada en el cmdline
        }

        my $class = $app->getAttributeNode("class")->getValue();

        if ($class eq "Cliente")
        {
            my %cliente_params;

            foreach my $param ($app->getChildNodes())
            {
                if ($param->getNodeType() != XML::DOM::ELEMENT_NODE)
                {
                    # Metadata de DOM
                    next;
                }

                if ($param->getTagName() eq "nr_paquetes")
                {
                    my $nr_paquetes = $param->getAttributeNode("nr_paquetes")->getValue();
                    
                    $cliente_params{nr_paquetes} = {nr_paquetes => $nr_paquetes};
                }
                elsif ($param->getTagName() eq "tiempo_cpu")
                {
                    my $distribucion = $param->getAttributeNode("distribucion")->getValue();
                    my $nominal = $param->getAttributeNode("nominal")->getValue();
                    my $rango = $param->getAttributeNode("rango")->getValue();

                    $cliente_params{tiempo_cpu} = {distribucion => $distribucion, nominal => $nominal, rango => $rango};
                }
                elsif ($param->getTagName() eq "periodo")
                {
                    my $distribucion = $param->getAttributeNode("distribucion")->getValue();
                    my $nominal = $param->getAttributeNode("nominal")->getValue();
                    my $rango = $param->getAttributeNode("rango")->getValue();

                    $cliente_params{periodo} = {distribucion => $distribucion, nominal => $nominal, rango => $rango};
                }
                elsif ($param->getTagName() eq "long_mensaje")
                {
                    my $distribucion = $param->getAttributeNode("distribucion")->getValue();
                    my $nominal = $param->getAttributeNode("nominal")->getValue();
                    my $rango = $param->getAttributeNode("rango")->getValue();

                    $cliente_params{long_mensaje} = {distribucion => $distribucion, nominal => $nominal, rango => $rango};
                }
                elsif ($param->getTagName() eq "destino")
                {
                    my $nodo = $param->getAttributeNode("nodo")->getValue();
                    my $listener = $param->getAttributeNode("listener")->getValue();

                    $cliente_params{destino} = {ip => $nodos_ip{$nodo}, puerto => $listener_port{$listener} };
                }
            }

            my $cmdline = construye_cliente_cmdline(\%cliente_params, $nombre);

            launch_cmdline($nombre, $cmdline);
        }
        elsif ($class eq "Servicio")
        {
            my %servicio_params;

            my $own_listener = $app->getAttributeNode("listener")->getValue();
            $servicio_params{acceso} = {puerto => $listener_port{$own_listener} };

            foreach my $param ($app->getChildNodes())
            {
                if ($param->getNodeType() != XML::DOM::ELEMENT_NODE)
                {
                    # Metadata de DOM
                    next;
                }

                if ($param->getTagName() eq "tiempo_cpu")
                {
                    my $distribucion = $param->getAttributeNode("distribucion")->getValue();
                    my $nominal = $param->getAttributeNode("nominal")->getValue();
                    my $rango = $param->getAttributeNode("rango")->getValue();


                    $servicio_params{tiempo_cpu} = {distribucion => $distribucion, nominal => $nominal, rango => $rango};
                }
                elsif ($param->getTagName() eq "modo")
                {
                    my $modo = $param->getAttributeNode("modo")->getValue();

                    $servicio_params{modo} = {modo => $modo};
                }
                elsif ($param->getTagName() eq "destino2")
                {
                    my $porcentage = $param->getAttributeNode("porcentage")->getValue();
                    my $nodo = $param->getAttributeNode("nodo")->getValue();
                    my $listener = $param->getAttributeNode("listener")->getValue();

                    $servicio_params{destino2} = {porcentage => $porcentage, ip => $nodos_ip{$nodo},
                                                   puerto => $listener_port{$listener}};
                }
                elsif ($param->getTagName() eq "destino1")
                {
                    my $porcentage = $param->getAttributeNode("porcentage")->getValue();
                    my $nodo = $param->getAttributeNode("nodo")->getValue();
                    my $listener = $param->getAttributeNode("listener")->getValue();

                    $servicio_params{destino1} = {porcentage => $porcentage, ip => $nodos_ip{$nodo},
                                                   puerto => $listener_port{$listener}};
                }
            }

            my $cmdline = construye_servicio_cmdline(\%servicio_params, $nombre);
            launch_cmdline($nombre, $cmdline);
        }
    }
}

$tree->dispose();


# ---------------------------------------
# F U N C I O N E S   D E   S O P O R T E
# ---------------------------------------


# -----------------------------------------------------------------------------

sub construye_cliente_cmdline
{
    my ($cliente_params, $nombre) = @_;

    my $cmdline = "java $java_defines -classpath $main_classdir:$swt_jar ";

    $cmdline .= " clienteservicio/Cliente -nombre $nombre ";

    if ( exists($cliente_params->{nr_paquetes}) )
    {
        $cmdline .= " -nr_paquetes " . join (",", ($cliente_params->{nr_paquetes}{nr_paquetes} ) );
    }

    if ( exists($cliente_params->{periodo}) )
    {
        $cmdline .= " -periodo " . join (",", ($cliente_params->{periodo}{distribucion},
                                                  $cliente_params->{periodo}{nominal},
                                                  $cliente_params->{periodo}{rango}) );
    }

    if ( exists($cliente_params->{tiempo_cpu}) )
    {
        $cmdline .= " -tiempo_cpu " . join (",",  ($cliente_params->{tiempo_cpu}{distribucion},
                                                 $cliente_params->{tiempo_cpu}{nominal},
                                                 $cliente_params->{tiempo_cpu}{rango}) );
    }

    if ( exists($cliente_params->{long_mensaje}) )
    {
        $cmdline .= " -long_mensaje " . join (",", ($cliente_params->{long_mensaje}{distribucion},
                                                $cliente_params->{long_mensaje}{nominal},
                                                $cliente_params->{long_mensaje}{rango}) );
    }

    if ( exists($cliente_params->{destino}) )
    {
        $cmdline .= " -destino " . join (",", ($cliente_params->{destino}{ip},
                                               $cliente_params->{destino}{puerto}) );
    }

    return $cmdline;
}

# ----------------------------------------------------------------------------------

sub construye_servicio_cmdline
{
    my ($servicio_params, $nombre) = @_;

    my $cmdline = "java $java_defines -classpath $main_classdir:$swt_jar ";

    $cmdline .= " clienteservicio/Servicio -nombre $nombre ";

    if ( exists($servicio_params->{acceso}) )
    {
        $cmdline .= " -acceso " . join (",",  ($servicio_params->{acceso}{puerto}) );
        
    }

    if ( exists($servicio_params->{modo}) )
    {
        $cmdline .= " -modo " . join(",", ($servicio_params->{modo}{modo}) );
    }

    if ( exists($servicio_params->{tiempo_cpu}) )
    {
        $cmdline .= " -tiempo_cpu " . join (",",  ($servicio_params->{tiempo_cpu}{distribucion},
                                                 $servicio_params->{tiempo_cpu}{nominal},
                                                 $servicio_params->{tiempo_cpu}{rango}) );

    }

    if ( exists($servicio_params->{destino1}) )
    {
        $cmdline .= " -destino1 " . join (",", ($servicio_params->{destino1}{porcentage},
                                                  $servicio_params->{destino1}{ip},
                                                  $servicio_params->{destino1}{puerto}) );
    }

    if ( exists($servicio_params->{destino2}) )
    {
        $cmdline .= " -destino2 " . join (",", ($servicio_params->{destino2}{porcentage},
                                                 $servicio_params->{destino2}{ip},
                                                 $servicio_params->{destino2}{puerto}) );
    }

    return $cmdline;
}

# -------------------------------------------------------

sub launch_cmdline()
{
    my ($name, $cmdline) = @_;

    # We add stdout & stderr redirection
    $cmdline .= ' | tee ' . "$name.output";

    print("LAUNCHER:  A punto de lanzar: $cmdline\n");
    my $task =  Proc::Background->new($cmdline);
    print("LAUNCHER:  Comando lanzado, PID " . $task->pid() . "\n");
}
