package tiposcomunes;

import java.util.ArrayList;
import java.util.Collections;
import java.util.EnumMap;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import cargasintetica.Carga;

public class LongComboData {
	
	// T H I S   I S   T H E   V A L U E
	public long value;
	public String nombre;
	public int ordinal = 0;
	
	// Note:  Maybe a lock might be needed if accessed concurrently
	private HashMap<String, Long> hash_map = new HashMap<String, Long>();
	private ArrayList<String> ordered_keys = new ArrayList<String>();

	private boolean regenerate_tags_pending = false;
	
	private enum Special_Values {CUSTOM, CALIBRATION};
	private EnumSet<Special_Values> special_present = EnumSet.noneOf(Special_Values.class);
	private EnumMap<Special_Values,String> special_keys = new EnumMap<Special_Values,String>(Special_Values.class);
	private EnumMap<Special_Values,Integer> special_ordinals = new EnumMap<Special_Values,Integer>(Special_Values.class);

	// Para la calibracion
	private Carga carga_a_calibrar = null;
	private long valor_calibracion;
	private Object autocalibracion_lock = new Object();
	private boolean autocalibracion_finalizada = false;
	
	public LongComboData(String nombre, Carga carga_a_calibrar)
	{
		this.nombre = nombre;
		this.carga_a_calibrar = carga_a_calibrar;
		value = -1;
		ordinal = -1;
		special_present.clear();
		special_ordinals.clear();
		special_keys.put(Special_Values.CUSTOM, "CUSTOM");
		special_keys.put(Special_Values.CALIBRATION, "CALIBRATION");
	}

	public void load_hashmap(HashMap<String,Long> my_hash_map)
	{
		// Ideally we could add
		Iterator<String> key_iterator = my_hash_map.keySet().iterator();
		
		while(key_iterator.hasNext())
		{
			String key = key_iterator.next();
			Long value = my_hash_map.get(key);
			
			hash_map.put(key, value);
		}
		
		regenerate_ordered_keys();
	}
	
	public List<String> exporta_ordered_keys()
	{
		
		ArrayList<String> res = new ArrayList<String>(ordered_keys);
		return res;
	}
	
	public boolean get_regenerate_tags_pending()
	{
		boolean res = regenerate_tags_pending;
		
		if (regenerate_tags_pending)
		{
			regenerate_tags_pending = false;
		}
		
		return res;
	}
	
	private int get_ordinal(long my_value)
	{
		Iterator<String> iterator = ordered_keys.iterator();
		
		boolean found = false;
		int position = -1;
		
		while (!found && iterator.hasNext())
		{
			String key = iterator.next();
			position++;
			if (hash_map.get(key) == my_value)
			{
				found = true;
				break;
			}
			
		}
		
		if (found)
		{
			return position;
		}

		return -1;
		
	}
	

	public void asigna_valor(long my_value)
	{
		ordinal = get_ordinal(my_value);
		
		if (ordinal == -1)
		{
			hash_map.put(special_keys.get(Special_Values.CUSTOM), my_value);
			if (special_present.contains(Special_Values.CUSTOM) )
			{
				ordinal = special_ordinals.get(Special_Values.CUSTOM);
			}
			else
			{
				special_present.add(Special_Values.CUSTOM);
				insert_special_value(Special_Values.CUSTOM, my_value);
				regenerate_ordered_keys();
				ordinal = special_ordinals.get(Special_Values.CUSTOM);
			}
		}

		value = my_value;
		
	}
	
	public void asigna_valor_por_id(int id)
	{
		if (id == -1)
		{
			// Esto fuerza una calibracion sin pasar por la GUI
			System.out.println("Autocalibracion");
			autocalibracion_finalizada = false;
			this.calibra(new StartupCalibrationHandler());
			
			synchronized (autocalibracion_lock)
			{
				while (! autocalibracion_finalizada)
				{
					try {
						autocalibracion_lock.wait();
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
			id = special_ordinals.get(Special_Values.CALIBRATION);
			
		}

		
		String key = ordered_keys.get(id);
		ordinal = id;
		value = hash_map.get(key);
	}

	
	
	public void parsea_cmdarg_ordinal(String arg)
	{
		int ordinal = Integer.valueOf(arg.trim()).intValue();
		

		asigna_valor_por_id(ordinal);
		
	}
	
	public void parsea_cmdarg_value(String arg)
	{
		long my_value = Long.valueOf(arg.trim()).longValue();
		
		asigna_valor(my_value);
	}

	private void regenerate_ordered_keys()
	{
		EnumMap<Special_Values,Long> special_values = new EnumMap<Special_Values,Long>(Special_Values.class);

		// First we remove the special values
		if (special_present.contains(Special_Values.CUSTOM))
		{
			special_values.put(Special_Values.CUSTOM, remove_special_value(Special_Values.CUSTOM) );
		}

		if (special_present.contains(Special_Values.CALIBRATION))
		{
			special_values.put(Special_Values.CALIBRATION, remove_special_value(Special_Values.CALIBRATION) );
		}
		
		// We clear the ordered_keys
		ordered_keys.clear();
		
		Iterator<String> iterator = hash_map.keySet().iterator();
		
		while(iterator.hasNext())
		{
			ordered_keys.add( iterator.next() );
		}
		
		Collections.sort(ordered_keys);
		
		// Now we add the special values
		if (special_present.contains(Special_Values.CUSTOM))
		{
			insert_special_value(Special_Values.CUSTOM, special_values.get(Special_Values.CUSTOM));
		}
		
		if (special_present.contains(Special_Values.CALIBRATION))
		{
			insert_special_value(Special_Values.CALIBRATION, special_values.get(Special_Values.CALIBRATION));
		}
		
		regenerate_tags_pending = true;
	}
	
	// Adds the special value to the hash_map and ordered_keys
	private void insert_special_value(Special_Values sp_value, long value)
	{
		String special_key = special_keys.get(sp_value);
		
		// We check that the special value is not included in the hash_map or in
		// ordered_keys
		assert( ! hash_map.containsKey(special_key) );
		// Don't bother to check ordered_keys
		
		// We add the value to the hash_map, combo_tag to ordered_keys (at the end) and record the ordinal
		hash_map.put(special_key, value);
		ordered_keys.add(special_key);
		special_ordinals.put(sp_value, ordered_keys.size() - 1);
	}

	private long remove_special_value(Special_Values sp_value)
	{
		String special_key = special_keys.get(sp_value);
		
		long res = hash_map.get(special_key);
		hash_map.remove(special_key);
		ordered_keys.remove(special_ordinals.get(sp_value));
		special_ordinals.remove(sp_value);
		
		return res;
	}

	// Callback is from the GUI
	public void calibra(CalibrationEndListener callback)
	{
		new CalibrationThread(callback).start();
	}

	private class CalibrationThread extends Thread
	{
		private CalibrationEndListener callback = null;
		CalibrationThread(CalibrationEndListener callback)
		{
			this.callback = callback;
		}
		
		public void run()
		{
			System.out.println("Comenzando calibracion");
			valor_calibracion = carga_a_calibrar.calibra(true, false);
			callback.finish_calibration_process();
			
		}
	}

	
	// This gets called from the GUI callback
	public void finish_calibration()
	{
		System.out.println("Calibracion_finalizada: " + valor_calibracion);

		// To reduce typing
		Special_Values sp_value = Special_Values.CALIBRATION;

		if (special_present.contains(sp_value) )
		{
			// Ya hemos calibrado antes y la key esta en su sitio
			hash_map.put(special_keys.get(sp_value), valor_calibracion);
		}
		else
		{
			// Esta es la primera calibracion
			special_present.add(sp_value);
			hash_map.put(special_keys.get(sp_value), valor_calibracion);
			regenerate_ordered_keys();
		}
		value = valor_calibracion;
		ordinal = special_ordinals.get(sp_value);
		System.out.println("Valor de calibracion: " + valor_calibracion);

	}

	private class StartupCalibrationHandler implements CalibrationEndListener
	{
		public void finish_calibration_process() 
		{
			finish_calibration();
			regenerate_tags_pending = true; // For the gui
			synchronized(autocalibracion_lock)
			{
				autocalibracion_finalizada = true;
				autocalibracion_lock.notify();
			}
		}
		
	}
}
