package tiposcomunes;

import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import clienteservicio.SWTUtil;

public class LongComboDataGUI implements ActionEnableDisableListener
{
	Group swt_group = null;
	Combo combo = null;
	Text valor = null;
	Button boton_calibrar = null;
	SelectionListener boton_calibrar_handler = null;
	Label mensaje_calibrar = null;
	
	LongComboData dato = null;
	private Shell shell;  // For defered asyncExec
	private ActionEnableDisableListener main_gui_aed_handler = null;

	public LongComboDataGUI(Shell shell, LongComboData dato, ActionEnableDisableListener main_gui_aed_handler)
	{
		this.shell = shell;
		this.main_gui_aed_handler = main_gui_aed_handler;
		swt_group = new Group(shell, SWT.SHADOW_OUT);
		swt_group.setText(dato.nombre);
		swt_group.setLayout(new GridLayout(2, false) );

		this.dato = dato;
		
		combo = new Combo(swt_group, SWT.NONE);
		combo.addSelectionListener(new ComboHandler());
//		GridData gridData_combo = new GridData();
//		gridData_combo.widthHint = 175;  // Copied from ve GUI version
//		combo.setLayoutData(gridData_combo);

		List<String> combo_tags = dato.exporta_ordered_keys();
		regenera_combo_tags(combo_tags);
		
		valor = new Text(swt_group, SWT.BORDER);
		// String with enough space for later values to fit in
		valor.setText("asdfasdf");
		valor.addSelectionListener(new TextHandler());
		
		boton_calibrar = new Button(swt_group, SWT.TOGGLE);
		boton_calibrar.setText("Calibrar");
		boton_calibrar_handler = new CalibrationStartHandler();
		boton_calibrar.addSelectionListener(boton_calibrar_handler );
		
		mensaje_calibrar = new Label(swt_group, SWT.NONE);
		GridData gridData_mensaje = new GridData();
		gridData_mensaje.widthHint = 80;
		mensaje_calibrar.setLayoutData(gridData_mensaje);
	}

	public void actualiza_valor()
	{
		combo.select(dato.ordinal);
		valor.setText(Long.toString(dato.value) );
	}

	
	public void lee_valor()
	{
		long res = Long.parseLong(valor.getText());
		dato.asigna_valor(res);
		if (dato.get_regenerate_tags_pending())
		{
			List<String> combo_tags = dato.exporta_ordered_keys();
			regenera_combo_tags(combo_tags);
		}

	}

	public void trigger_calibration()
	{
		SelectionEvent evento_start = new SelectionEvent(new Event());
		
		boton_calibrar_handler.widgetDefaultSelected(evento_start);
	}
	
	private void regenera_combo_tags(List<String> tags)
	{
		combo.removeAll();
		Iterator<String> iterator = tags.iterator();
		
		while (iterator.hasNext() )
		{
			combo.add(iterator.next());
		}

		
	}
	
	
	private class ComboHandler implements SelectionListener
	{
		public void widgetSelected(SelectionEvent arg0)
		{
			int ordinal = combo.getSelectionIndex();
			dato.asigna_valor_por_id(ordinal);
			valor.setText(Long.toString(dato.value) );
			
		}
		
		// Typically called with double click
		public void widgetDefaultSelected(SelectionEvent arg0) 
		{
			// Do nothing (just to distinguish it)
			System.out.println("DEFAULT EVENT");
			return;
			// widgetSelected(arg0)
		}

		
	}
	
	private class TextHandler implements SelectionListener
	{

		public void widgetDefaultSelected(SelectionEvent arg0) {
			long my_value = Long.parseLong(valor.getText() );
			
			dato.asigna_valor(my_value);
			
			if (dato.get_regenerate_tags_pending())
			{
				List<String> combo_tags = dato.exporta_ordered_keys();
				regenera_combo_tags(combo_tags);
			}

			
			combo.select(dato.ordinal);
			
		}

		public void widgetSelected(SelectionEvent arg0) {
			System.out.println("SELECTED");
			
		}

		
	}
	
	
	// Since calibration takes time, we cannot leave the GUI unresponsive
	// and frozen during the process.  Therefore we delegate the calibration work to
	// a external thread and pass the callibration end listener callback to it.
	private class CalibrationStartHandler implements SelectionListener
	{

		public void widgetDefaultSelected(SelectionEvent arg0) {
			System.out.println("DefaultSelected");
		}

		public void widgetSelected(SelectionEvent arg0) {
			boton_calibrar.setEnabled(false);
			main_gui_aed_handler.disable_actions();
			mensaje_calibrar.setText("Calibrando...");
			dato.calibra(new CalibrationEndHandler());
		}
	}
	
	private class CalibrationEndHandler implements CalibrationEndListener
	{
		public void finish_calibration_process()
		{
			dato.finish_calibration();

			// Este metodo lo llama el thread calibrador que NO puede
			// modificar elementos de la GUI directamente.
			
			// En su lugar delega en el método run() de un thread anónimo pasado
			// por asynExec() para que se ejecute "en cuanto sea posible".
			Display display = shell.getDisplay();
			
			display.asyncExec( new Runnable() 
			{
				public void run()
				{
					mensaje_calibrar.setText("");
					if (dato.get_regenerate_tags_pending())
					{
						List<String> combo_tags = dato.exporta_ordered_keys();
						regenera_combo_tags(combo_tags);
					}
					combo.select(dato.ordinal);
					valor.setText(Long.toString(dato.value) );
					boton_calibrar.setEnabled(true);
					main_gui_aed_handler.enable_actions();
					
				}
			});
			
		}
		
	}

	// ActionEnableDisableListener
	public void disable_actions() 
	{
		boton_calibrar.setEnabled(false);
		
	}

	public void enable_actions() {
		boton_calibrar.setEnabled(true);
		
	}
	
	
}
